//********************************************************************
//  Account3.java       Author: Lewis and Loftus
//
//  Solution to Programming Project 6.9 (5E, p. 268)
//********************************************************************

import java.text.NumberFormat;

public class Account3 implements Lockable
{
   private NumberFormat fmt = NumberFormat.getCurrencyInstance();

   private final double RATE = 0.035;  // interest rate of 3.5%

   private long acctNumber;
   private double balance;
   private String name;

   private int accountKey;
   private boolean locked;

   //-----------------------------------------------------------------
   //  Sets up the account by defining its owner and account number.
   //  Initial balance is set to zero
   //-----------------------------------------------------------------
   public Account3 (String owner, long account)
   {
      name = owner;
      acctNumber = account;
      balance = 0.0;
      locked = false;
      accountKey = 0;
   }

   //-----------------------------------------------------------------
   //  Sets up the account by defining its owner, account number,
   //  and initial balance.
   //-----------------------------------------------------------------
   public Account3 (String owner, long account, double initial)
   {
      name = owner;
      acctNumber = account;
      balance = initial;
      locked = false;
      accountKey = 0;
   }

   //-----------------------------------------------------------------
   //  Validates the transaction.  If sufficent funds exist, withdraws
   //  the specified amount from the "from" account and deposits it
   //  into the "to" account returning true, else does nothing.
   //-----------------------------------------------------------------
   public static boolean transfer (double amount, double fee, Account3
	      from, Account3 to)
   {

      if (from.locked() || to.locked())
         return false;

      if (from.balance + fee < amount || amount < 0)
         return false;

      from.withdraw(amount, fee);
      to.deposit(amount);

      return true;
   }

   //-----------------------------------------------------------------
   //  Validates the transaction, then deposits the specified amount
   //  into the account. Returns the new balance.
   //-----------------------------------------------------------------
   public double deposit (double amount)
   {
      if (locked)
         return balance;

      if (amount < 0)  // deposit value is negative
      {
         System.out.println ();
         System.out.println ("Error: Deposit amount is invalid.");
         System.out.println (acctNumber + "  " + fmt.format(amount));
      }
      else
         balance = balance + amount;

      return balance;
   }

   //-----------------------------------------------------------------
   //  Validates the transaction, then withdraws the specified amount
   //  from the account. Returns the new balance.
   //-----------------------------------------------------------------
   public double withdraw (double amount, double fee)
   {

      if (locked)
         return balance;

      amount += fee;

      if (amount < 0)  // withdraw value is negative
      {
         System.out.println ();
         System.out.println ("Error: Withdraw amount is invalid.");
         System.out.println ("Account: " + acctNumber);
         System.out.println ("Requested: " + fmt.format(amount));
      }
      else
         if (amount > balance)  // withdraw value exceeds balance
         {
            System.out.println ();
            System.out.println ("Error: Insufficient funds.");
            System.out.println ("Account: " + acctNumber);
            System.out.println ("Requested: " + fmt.format(amount));
            System.out.println ("Available: " + fmt.format(balance));
         }
         else
            balance = balance - amount;

      return balance;
   }

   //-----------------------------------------------------------------
   //  Adds interest to the account and returns the new balance.
   //-----------------------------------------------------------------
   public double addInterest ()
   {
      if (locked)
         return balance;

      balance += (balance * RATE);
      return balance;
   }

   //-----------------------------------------------------------------
   //  Returns the current balance of the account.
   //-----------------------------------------------------------------
   public double getBalance ()
   {
      return balance;
   }

   //-----------------------------------------------------------------
   //  Returns the account number.
   //-----------------------------------------------------------------
   public long getAccountNumber ()
   {
      return acctNumber;
   }

   //-----------------------------------------------------------------
   //  Returns a one-line description of the account as a string.
   //-----------------------------------------------------------------
   public String toString ()
   {
      return (acctNumber + "\t" + name + "\t" + fmt.format(balance));
   }

   //-----------------------------------------------------------------
   //  Sets the account's key
   //-----------------------------------------------------------------
   public void setKey (int key)
   {
      accountKey = key;
   }

   //-----------------------------------------------------------------
   //  Locks the account
   //-----------------------------------------------------------------
   public void lock (int key)
   {
      if (key == accountKey)
         locked = true;
   }

   //-----------------------------------------------------------------
   //  Unlocks the account
   //-----------------------------------------------------------------
   public void unlock (int key)
   {
      if (key == accountKey)
         locked = false;
   }

   //-----------------------------------------------------------------
   //  Returns true it the account is locked, else returns false
   //-----------------------------------------------------------------
   public boolean locked()
   {
      return locked;
   }
}
